@@ -296,7 +296,7 @@ struct statistics
   double time;
   unsigned states;
   unsigned edges;
-  unsigned transitions;
+  unsigned long long transitions;
   unsigned acc;
   unsigned scc;
   unsigned nonacc_scc;
@@ -1336,25 +1336,37 @@ namespace
                           && !spot::is_universal(from[i]))
                         missing_complement = true;
                       else
-                        to[i] = spot::complement(from[i], aborter);
-                      if (verbose)
-                        {
-                          if (to[i])
-                            {
-                              std::cerr << "\t(";
-                              printsize(from[i]);
-                              std::cerr << ") -> (";
-                              printsize(to[i]);
-                              std::cerr << ")\tComp(" << prefix << i << ")\n";
-                            }
-                          else
-                            {
-                              std::cerr << "\tnot complemented";
-                              if (aborter)
-                                aborter->print_reason(std::cerr << " (") << ')';
-                              std::cerr << '\n';
-                            }
-                        }
+                        try
+                          {
+                            to[i] = spot::complement(from[i], aborter);
+                            if (verbose)
+                              {
+                                if (to[i])
+                                  {
+                                    std::cerr << "\t(";
+                                    printsize(from[i]);
+                                    std::cerr << ") -> (";
+                                    printsize(to[i]);
+                                    std::cerr << ")\tComp(" << prefix
+                                              << i << ")\n";
+                                  }
+                                else
+                                  {
+                                    std::cerr << "\tnot complemented";
+                                    if (aborter)
+                                      aborter->print_reason(std::cerr
+                                                            << " (") << ')';
+                                    std::cerr << '\n';
+                                  }
+                              }
+                          }
+                        catch (const std::runtime_error& re)
+                          {
+                            missing_complement = true;
+                            if (verbose)
+                              std::cerr << "\tnot complemented ("
+                                        << re.what() << ")\n";
+                          }
                     }
                 };
               missing_complement = false;
@@ -1519,6 +1531,7 @@ namespace
                                   << " ed.\n";
                       sm = new
                         spot::scc_info(p, spot::scc_info_options::TRACK_STATES);
+                      sm->determine_unknown_acceptance();
                     }
                   catch (const std::bad_alloc&)
                     {
@@ -1556,6 +1569,7 @@ namespace
                         sm = new
                           spot::scc_info(p,
                                          spot::scc_info_options::TRACK_STATES);
+                        sm->determine_unknown_acceptance();
                       }
                     catch (const std::bad_alloc&)
                       {
