@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016, 2018 Laboratoire de Recherche
+// Copyright (C) 2012-2016, 2018-2019 Laboratoire de Recherche
 // et Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -159,6 +159,8 @@ static bool ap_count_given = false;
 static int
 parse_opt(int key, char* arg, struct argp_state* as)
 {
+  // Called from C code, so should not raise any exception.
+  BEGIN_EXCEPTION_PROTECT;
   // This switch is alphabetically-ordered.
   switch (key)
     {
@@ -231,6 +233,7 @@ parse_opt(int key, char* arg, struct argp_state* as)
     default:
       return ARGP_ERR_UNKNOWN;
     }
+  END_EXCEPTION_PROTECT;
   return 0;
 }
 
