@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.3.2.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,11 +41,14 @@
    define necessary library symbols; they are noted "INFRINGES ON
    USER NAME SPACE" below.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 /* Identify Bison output.  */
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.0.4"
+#define YYBISON_VERSION "3.3.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -61,8 +65,8 @@
 
 
 
-/* Copy the first part of user declarations.  */
-#line 8 "parser.y" /* yacc.c:339  */
+/* First part of user prologue.  */
+#line 8 "parser.y" /* yacc.c:337  */
 
 #include <string.h>
 #include <stdarg.h>
@@ -119,13 +123,16 @@ void actQuantVar1(token *res, token *id);
 void actPrint(token *id);
 
 
-#line 123 "parser.c" /* yacc.c:339  */
-
+#line 127 "parser.c" /* yacc.c:337  */
 # ifndef YY_NULLPTR
-#  if defined __cplusplus && 201103L <= __cplusplus
-#   define YY_NULLPTR nullptr
+#  if defined __cplusplus
+#   if 201103L <= __cplusplus
+#    define YY_NULLPTR nullptr
+#   else
+#    define YY_NULLPTR 0
+#   endif
 #  else
-#   define YY_NULLPTR 0
+#   define YY_NULLPTR ((void*)0)
 #  endif
 # endif
 
@@ -242,9 +249,7 @@ int yyparse (void);
 
 #endif /* !YY_YY_PARSER_H_INCLUDED  */
 
-/* Copy the second part of user declarations.  */
 
-#line 248 "parser.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -265,13 +270,13 @@ typedef signed char yytype_int8;
 #ifdef YYTYPE_UINT16
 typedef YYTYPE_UINT16 yytype_uint16;
 #else
-typedef unsigned short int yytype_uint16;
+typedef unsigned short yytype_uint16;
 #endif
 
 #ifdef YYTYPE_INT16
 typedef YYTYPE_INT16 yytype_int16;
 #else
-typedef short int yytype_int16;
+typedef short yytype_int16;
 #endif
 
 #ifndef YYSIZE_T
@@ -283,7 +288,7 @@ typedef short int yytype_int16;
 #  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
 #  define YYSIZE_T size_t
 # else
-#  define YYSIZE_T unsigned int
+#  define YYSIZE_T unsigned
 # endif
 #endif
 
@@ -319,15 +324,6 @@ typedef short int yytype_int16;
 # define YY_ATTRIBUTE_UNUSED YY_ATTRIBUTE ((__unused__))
 #endif
 
-#if !defined _Noreturn \
-     && (!defined __STDC_VERSION__ || __STDC_VERSION__ < 201112)
-# if defined _MSC_VER && 1200 <= _MSC_VER
-#  define _Noreturn __declspec (noreturn)
-# else
-#  define _Noreturn YY_ATTRIBUTE ((__noreturn__))
-# endif
-#endif
-
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
 # define YYUSE(E) ((void) (E))
@@ -335,7 +331,7 @@ typedef short int yytype_int16;
 # define YYUSE(E) /* empty */
 #endif
 
-#if defined __GNUC__ && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
+#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
 /* Suppress an incorrect diagnostic about yylval being uninitialized.  */
 # define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN \
     _Pragma ("GCC diagnostic push") \
@@ -497,16 +493,16 @@ union yyalloc
 /* YYNSTATES -- Number of states.  */
 #define YYNSTATES  84
 
-/* YYTRANSLATE[YYX] -- Symbol number corresponding to YYX as returned
-   by yylex, with out-of-bounds checking.  */
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   292
 
+/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
+   as returned by yylex, with out-of-bounds checking.  */
 #define YYTRANSLATE(YYX)                                                \
-  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
+  ((unsigned) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
-   as returned by yylex, without out-of-bounds checking.  */
+   as returned by yylex.  */
 static const yytype_uint8 yytranslate[] =
 {
        0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
@@ -714,22 +710,22 @@ static const yytype_uint8 yyr2[] =
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
-#define YYBACKUP(Token, Value)                                  \
-do                                                              \
-  if (yychar == YYEMPTY)                                        \
-    {                                                           \
-      yychar = (Token);                                         \
-      yylval = (Value);                                         \
-      YYPOPSTACK (yylen);                                       \
-      yystate = *yyssp;                                         \
-      goto yybackup;                                            \
-    }                                                           \
-  else                                                          \
-    {                                                           \
-      yyerror (YY_("syntax error: cannot back up")); \
-      YYERROR;                                                  \
-    }                                                           \
-while (0)
+#define YYBACKUP(Token, Value)                                    \
+  do                                                              \
+    if (yychar == YYEMPTY)                                        \
+      {                                                           \
+        yychar = (Token);                                         \
+        yylval = (Value);                                         \
+        YYPOPSTACK (yylen);                                       \
+        yystate = *yyssp;                                         \
+        goto yybackup;                                            \
+      }                                                           \
+    else                                                          \
+      {                                                           \
+        yyerror (YY_("syntax error: cannot back up")); \
+        YYERROR;                                                  \
+      }                                                           \
+  while (0)
 
 /* Error token number */
 #define YYTERROR        1
@@ -769,37 +765,37 @@ do {                                                                      \
 } while (0)
 
 
-/*----------------------------------------.
-| Print this symbol's value on YYOUTPUT.  |
-`----------------------------------------*/
+/*-----------------------------------.
+| Print this symbol's value on YYO.  |
+`-----------------------------------*/
 
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_value_print (FILE *yyo, int yytype, YYSTYPE const * const yyvaluep)
 {
-  FILE *yyo = yyoutput;
-  YYUSE (yyo);
+  FILE *yyoutput = yyo;
+  YYUSE (yyoutput);
   if (!yyvaluep)
     return;
 # ifdef YYPRINT
   if (yytype < YYNTOKENS)
-    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
+    YYPRINT (yyo, yytoknum[yytype], *yyvaluep);
 # endif
   YYUSE (yytype);
 }
 
 
-/*--------------------------------.
-| Print this symbol on YYOUTPUT.  |
-`--------------------------------*/
+/*---------------------------.
+| Print this symbol on YYO.  |
+`---------------------------*/
 
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
+yy_symbol_print (FILE *yyo, int yytype, YYSTYPE const * const yyvaluep)
 {
-  YYFPRINTF (yyoutput, "%s %s (",
+  YYFPRINTF (yyo, "%s %s (",
              yytype < YYNTOKENS ? "token" : "nterm", yytname[yytype]);
 
-  yy_symbol_value_print (yyoutput, yytype, yyvaluep);
-  YYFPRINTF (yyoutput, ")");
+  yy_symbol_value_print (yyo, yytype, yyvaluep);
+  YYFPRINTF (yyo, ")");
 }
 
 /*------------------------------------------------------------------.
@@ -833,7 +829,7 @@ do {                                                            \
 static void
 yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
 {
-  unsigned long int yylno = yyrline[yyrule];
+  unsigned long yylno = yyrline[yyrule];
   int yynrhs = yyr2[yyrule];
   int yyi;
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu):\n",
@@ -844,7 +840,7 @@ yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
       YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr,
                        yystos[yyssp[yyi + 1 - yynrhs]],
-                       &(yyvsp[(yyi + 1) - (yynrhs)])
+                       &yyvsp[(yyi + 1) - (yynrhs)]
                                               );
       YYFPRINTF (stderr, "\n");
     }
@@ -948,7 +944,10 @@ yytnamerr (char *yyres, const char *yystr)
           case '\\':
             if (*++yyp != '\\')
               goto do_not_strip_quotes;
-            /* Fall through.  */
+            else
+              goto append;
+
+          append:
           default:
             if (yyres)
               yyres[yyn] = *yyp;
@@ -966,7 +965,7 @@ yytnamerr (char *yyres, const char *yystr)
   if (! yyres)
     return yystrlen (yystr);
 
-  return yystpcpy (yyres, yystr) - yyres;
+  return (YYSIZE_T) (yystpcpy (yyres, yystr) - yyres);
 }
 # endif
 
@@ -1044,10 +1043,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                 yyarg[yycount++] = yytname[yyx];
                 {
                   YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULLPTR, yytname[yyx]);
-                  if (! (yysize <= yysize1
-                         && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
+                    yysize = yysize1;
+                  else
                     return 2;
-                  yysize = yysize1;
                 }
               }
         }
@@ -1059,6 +1058,7 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
       case N:                               \
         yyformat = S;                       \
       break
+    default: /* Avoid compiler warnings. */
       YYCASE_(0, YY_("syntax error"));
       YYCASE_(1, YY_("syntax error, unexpected %s"));
       YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
@@ -1070,9 +1070,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 
   {
     YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
-    if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
+      yysize = yysize1;
+    else
       return 2;
-    yysize = yysize1;
   }
 
   if (*yymsg_alloc < yysize)
@@ -1198,23 +1199,31 @@ yyparse (void)
   yychar = YYEMPTY; /* Cause a token to be read.  */
   goto yysetstate;
 
+
 /*------------------------------------------------------------.
-| yynewstate -- Push a new state, which is found in yystate.  |
+| yynewstate -- push a new state, which is found in yystate.  |
 `------------------------------------------------------------*/
- yynewstate:
+yynewstate:
   /* In all cases, when you get here, the value and location stacks
      have just been pushed.  So pushing a state here evens the stacks.  */
   yyssp++;
 
- yysetstate:
-  *yyssp = yystate;
+
+/*--------------------------------------------------------------------.
+| yynewstate -- set current state (the top of the stack) to yystate.  |
+`--------------------------------------------------------------------*/
+yysetstate:
+  *yyssp = (yytype_int16) yystate;
 
   if (yyss + yystacksize - 1 <= yyssp)
+#if !defined yyoverflow && !defined YYSTACK_RELOCATE
+    goto yyexhaustedlab;
+#else
     {
       /* Get the current used size of the three stacks, in elements.  */
-      YYSIZE_T yysize = yyssp - yyss + 1;
+      YYSIZE_T yysize = (YYSIZE_T) (yyssp - yyss + 1);
 
-#ifdef yyoverflow
+# if defined yyoverflow
       {
         /* Give user a chance to reallocate the stack.  Use copies of
            these so that the &'s don't force the real ones into
@@ -1230,14 +1239,10 @@ yyparse (void)
                     &yyss1, yysize * sizeof (*yyssp),
                     &yyvs1, yysize * sizeof (*yyvsp),
                     &yystacksize);
-
         yyss = yyss1;
         yyvs = yyvs1;
       }
-#else /* no yyoverflow */
-# ifndef YYSTACK_RELOCATE
-      goto yyexhaustedlab;
-# else
+# else /* defined YYSTACK_RELOCATE */
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
         goto yyexhaustedlab;
@@ -1253,22 +1258,22 @@ yyparse (void)
           goto yyexhaustedlab;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
         YYSTACK_RELOCATE (yyvs_alloc, yyvs);
-#  undef YYSTACK_RELOCATE
+# undef YYSTACK_RELOCATE
         if (yyss1 != yyssa)
           YYSTACK_FREE (yyss1);
       }
 # endif
-#endif /* no yyoverflow */
 
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
-                  (unsigned long int) yystacksize));
+                  (unsigned long) yystacksize));
 
       if (yyss + yystacksize - 1 <= yyssp)
         YYABORT;
     }
+#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
@@ -1277,11 +1282,11 @@ yyparse (void)
 
   goto yybackup;
 
+
 /*-----------.
 | yybackup.  |
 `-----------*/
 yybackup:
-
   /* Do appropriate processing given the current state.  Read a
      lookahead token if we need one and don't already have one.  */
 
@@ -1354,7 +1359,7 @@ yydefault:
 
 
 /*-----------------------------.
-| yyreduce -- Do a reduction.  |
+| yyreduce -- do a reduction.  |
 `-----------------------------*/
 yyreduce:
   /* yyn is the number of a rule to reduce with.  */
@@ -1375,211 +1380,211 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-#line 102 "parser.y" /* yacc.c:1646  */
-    { actInit(&(yyvsp[-2]),&(yyvsp[-1])); }
-#line 1381 "parser.c" /* yacc.c:1646  */
+#line 102 "parser.y" /* yacc.c:1652  */
+    { actInit(&yyvsp[-2],&yyvsp[-1]); }
+#line 1386 "parser.c" /* yacc.c:1652  */
     break;
 
   case 4:
-#line 106 "parser.y" /* yacc.c:1646  */
+#line 106 "parser.y" /* yacc.c:1652  */
     { actInputs(); }
-#line 1387 "parser.c" /* yacc.c:1646  */
+#line 1392 "parser.c" /* yacc.c:1652  */
     break;
 
   case 5:
-#line 110 "parser.y" /* yacc.c:1646  */
-    { actAddInput(&(yyvsp[0])); }
-#line 1393 "parser.c" /* yacc.c:1646  */
+#line 110 "parser.y" /* yacc.c:1652  */
+    { actAddInput(&yyvsp[0]); }
+#line 1398 "parser.c" /* yacc.c:1652  */
     break;
 
   case 6:
-#line 111 "parser.y" /* yacc.c:1646  */
-    { actAddInput(&(yyvsp[0])); }
-#line 1399 "parser.c" /* yacc.c:1646  */
+#line 111 "parser.y" /* yacc.c:1652  */
+    { actAddInput(&yyvsp[0]); }
+#line 1404 "parser.c" /* yacc.c:1652  */
     break;
 
   case 17:
-#line 137 "parser.y" /* yacc.c:1646  */
-    { actAssign(&(yyvsp[-2]),&(yyvsp[0])); }
-#line 1405 "parser.c" /* yacc.c:1646  */
+#line 137 "parser.y" /* yacc.c:1652  */
+    { actAssign(&yyvsp[-2],&yyvsp[0]); }
+#line 1410 "parser.c" /* yacc.c:1652  */
     break;
 
   case 18:
-#line 141 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_and); }
-#line 1411 "parser.c" /* yacc.c:1646  */
+#line 141 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_and); }
+#line 1416 "parser.c" /* yacc.c:1652  */
     break;
 
   case 19:
-#line 142 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_nand); }
-#line 1417 "parser.c" /* yacc.c:1646  */
+#line 142 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_nand); }
+#line 1422 "parser.c" /* yacc.c:1652  */
     break;
 
   case 20:
-#line 143 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_xor); }
-#line 1423 "parser.c" /* yacc.c:1646  */
+#line 143 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_xor); }
+#line 1428 "parser.c" /* yacc.c:1652  */
     break;
 
   case 21:
-#line 144 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_or); }
-#line 1429 "parser.c" /* yacc.c:1646  */
+#line 144 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_or); }
+#line 1434 "parser.c" /* yacc.c:1652  */
     break;
 
   case 22:
-#line 145 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_nor); }
-#line 1435 "parser.c" /* yacc.c:1646  */
+#line 145 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_nor); }
+#line 1440 "parser.c" /* yacc.c:1652  */
     break;
 
   case 23:
-#line 146 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_imp); }
-#line 1441 "parser.c" /* yacc.c:1646  */
+#line 146 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_imp); }
+#line 1446 "parser.c" /* yacc.c:1652  */
     break;
 
   case 24:
-#line 147 "parser.y" /* yacc.c:1646  */
-    { actOpr2(&(yyval),&(yyvsp[-2]),&(yyvsp[0]),bddop_biimp); }
-#line 1447 "parser.c" /* yacc.c:1646  */
+#line 147 "parser.y" /* yacc.c:1652  */
+    { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_biimp); }
+#line 1452 "parser.c" /* yacc.c:1652  */
     break;
 
   case 25:
-#line 148 "parser.y" /* yacc.c:1646  */
-    { actNot(&(yyval),&(yyvsp[0])); }
-#line 1453 "parser.c" /* yacc.c:1646  */
+#line 148 "parser.y" /* yacc.c:1652  */
+    { actNot(&yyval,&yyvsp[0]); }
+#line 1458 "parser.c" /* yacc.c:1652  */
     break;
 
   case 26:
-#line 149 "parser.y" /* yacc.c:1646  */
-    { (yyval).bval = (yyvsp[-1]).bval; }
-#line 1459 "parser.c" /* yacc.c:1646  */
+#line 149 "parser.y" /* yacc.c:1652  */
+    { yyval.bval = yyvsp[-1].bval; }
+#line 1464 "parser.c" /* yacc.c:1652  */
     break;
 
   case 27:
-#line 150 "parser.y" /* yacc.c:1646  */
-    { actId(&(yyval),&(yyvsp[0])); }
-#line 1465 "parser.c" /* yacc.c:1646  */
+#line 150 "parser.y" /* yacc.c:1652  */
+    { actId(&yyval,&yyvsp[0]); }
+#line 1470 "parser.c" /* yacc.c:1652  */
     break;
 
   case 28:
-#line 151 "parser.y" /* yacc.c:1646  */
-    { (yyval).bval = new bdd(bddtrue); }
-#line 1471 "parser.c" /* yacc.c:1646  */
+#line 151 "parser.y" /* yacc.c:1652  */
+    { yyval.bval = new bdd(bddtrue); }
+#line 1476 "parser.c" /* yacc.c:1652  */
     break;
 
   case 29:
-#line 152 "parser.y" /* yacc.c:1646  */
-    { (yyval).bval = new bdd(bddfalse); }
-#line 1477 "parser.c" /* yacc.c:1646  */
+#line 152 "parser.y" /* yacc.c:1652  */
+    { yyval.bval = new bdd(bddfalse); }
+#line 1482 "parser.c" /* yacc.c:1652  */
     break;
 
   case 30:
-#line 153 "parser.y" /* yacc.c:1646  */
-    { (yyval).bval = (yyvsp[0]).bval; }
-#line 1483 "parser.c" /* yacc.c:1646  */
+#line 153 "parser.y" /* yacc.c:1652  */
+    { yyval.bval = yyvsp[0].bval; }
+#line 1488 "parser.c" /* yacc.c:1652  */
     break;
 
   case 31:
-#line 157 "parser.y" /* yacc.c:1646  */
-    { actExist(&(yyval),&(yyvsp[-2]),&(yyvsp[0])); }
-#line 1489 "parser.c" /* yacc.c:1646  */
+#line 157 "parser.y" /* yacc.c:1652  */
+    { actExist(&yyval,&yyvsp[-2],&yyvsp[0]); }
+#line 1494 "parser.c" /* yacc.c:1652  */
     break;
 
   case 32:
-#line 158 "parser.y" /* yacc.c:1646  */
-    { actForall(&(yyval),&(yyvsp[-2]),&(yyvsp[0])); }
-#line 1495 "parser.c" /* yacc.c:1646  */
+#line 158 "parser.y" /* yacc.c:1652  */
+    { actForall(&yyval,&yyvsp[-2],&yyvsp[0]); }
+#line 1500 "parser.c" /* yacc.c:1652  */
     break;
 
   case 33:
-#line 162 "parser.y" /* yacc.c:1646  */
-    { actQuantVar2(&(yyval),&(yyvsp[0]),&(yyvsp[-1])); }
-#line 1501 "parser.c" /* yacc.c:1646  */
+#line 162 "parser.y" /* yacc.c:1652  */
+    { actQuantVar2(&yyval,&yyvsp[0],&yyvsp[-1]); }
+#line 1506 "parser.c" /* yacc.c:1652  */
     break;
 
   case 34:
-#line 163 "parser.y" /* yacc.c:1646  */
-    { actQuantVar1(&(yyval),&(yyvsp[0])); }
-#line 1507 "parser.c" /* yacc.c:1646  */
+#line 163 "parser.y" /* yacc.c:1652  */
+    { actQuantVar1(&yyval,&yyvsp[0]); }
+#line 1512 "parser.c" /* yacc.c:1652  */
     break;
 
   case 35:
-#line 168 "parser.y" /* yacc.c:1646  */
-    { actSize(&(yyvsp[0])); }
-#line 1513 "parser.c" /* yacc.c:1646  */
+#line 168 "parser.y" /* yacc.c:1652  */
+    { actSize(&yyvsp[0]); }
+#line 1518 "parser.c" /* yacc.c:1652  */
     break;
 
   case 36:
-#line 172 "parser.y" /* yacc.c:1646  */
-    { actDot(&(yyvsp[-1]),&(yyvsp[0])); }
-#line 1519 "parser.c" /* yacc.c:1646  */
+#line 172 "parser.y" /* yacc.c:1652  */
+    { actDot(&yyvsp[-1],&yyvsp[0]); }
+#line 1524 "parser.c" /* yacc.c:1652  */
     break;
 
   case 37:
-#line 176 "parser.y" /* yacc.c:1646  */
-    { bdd_reorder((yyvsp[0]).ival); }
-#line 1525 "parser.c" /* yacc.c:1646  */
+#line 176 "parser.y" /* yacc.c:1652  */
+    { bdd_reorder(yyvsp[0].ival); }
+#line 1530 "parser.c" /* yacc.c:1652  */
     break;
 
   case 38:
-#line 177 "parser.y" /* yacc.c:1646  */
-    { actAutoreorder(&(yyvsp[-1]),&(yyvsp[0])); }
-#line 1531 "parser.c" /* yacc.c:1646  */
+#line 177 "parser.y" /* yacc.c:1652  */
+    { actAutoreorder(&yyvsp[-1],&yyvsp[0]); }
+#line 1536 "parser.c" /* yacc.c:1652  */
     break;
 
   case 39:
-#line 181 "parser.y" /* yacc.c:1646  */
-    { (yyval).ival = BDD_REORDER_WIN2; }
-#line 1537 "parser.c" /* yacc.c:1646  */
+#line 181 "parser.y" /* yacc.c:1652  */
+    { yyval.ival = BDD_REORDER_WIN2; }
+#line 1542 "parser.c" /* yacc.c:1652  */
     break;
 
   case 40:
-#line 182 "parser.y" /* yacc.c:1646  */
-    { (yyval).ival = BDD_REORDER_WIN2ITE; }
-#line 1543 "parser.c" /* yacc.c:1646  */
+#line 182 "parser.y" /* yacc.c:1652  */
+    { yyval.ival = BDD_REORDER_WIN2ITE; }
+#line 1548 "parser.c" /* yacc.c:1652  */
     break;
 
   case 41:
-#line 183 "parser.y" /* yacc.c:1646  */
-    { (yyval).ival = BDD_REORDER_SIFT; }
-#line 1549 "parser.c" /* yacc.c:1646  */
+#line 183 "parser.y" /* yacc.c:1652  */
+    { yyval.ival = BDD_REORDER_SIFT; }
+#line 1554 "parser.c" /* yacc.c:1652  */
     break;
 
   case 42:
-#line 184 "parser.y" /* yacc.c:1646  */
-    { (yyval).ival = BDD_REORDER_SIFTITE; }
-#line 1555 "parser.c" /* yacc.c:1646  */
+#line 184 "parser.y" /* yacc.c:1652  */
+    { yyval.ival = BDD_REORDER_SIFTITE; }
+#line 1560 "parser.c" /* yacc.c:1652  */
     break;
 
   case 43:
-#line 185 "parser.y" /* yacc.c:1646  */
-    { (yyval).ival = BDD_REORDER_NONE; }
-#line 1561 "parser.c" /* yacc.c:1646  */
+#line 185 "parser.y" /* yacc.c:1652  */
+    { yyval.ival = BDD_REORDER_NONE; }
+#line 1566 "parser.c" /* yacc.c:1652  */
     break;
 
   case 44:
-#line 189 "parser.y" /* yacc.c:1646  */
+#line 189 "parser.y" /* yacc.c:1652  */
     { actCache(); }
-#line 1567 "parser.c" /* yacc.c:1646  */
+#line 1572 "parser.c" /* yacc.c:1652  */
     break;
 
   case 45:
-#line 193 "parser.y" /* yacc.c:1646  */
-    { actTautology(&(yyvsp[0])); }
-#line 1573 "parser.c" /* yacc.c:1646  */
+#line 193 "parser.y" /* yacc.c:1652  */
+    { actTautology(&yyvsp[0]); }
+#line 1578 "parser.c" /* yacc.c:1652  */
     break;
 
   case 46:
-#line 197 "parser.y" /* yacc.c:1646  */
-    { actPrint(&(yyvsp[0])); }
-#line 1579 "parser.c" /* yacc.c:1646  */
+#line 197 "parser.y" /* yacc.c:1652  */
+    { actPrint(&yyvsp[0]); }
+#line 1584 "parser.c" /* yacc.c:1652  */
     break;
 
 
-#line 1583 "parser.c" /* yacc.c:1646  */
+#line 1588 "parser.c" /* yacc.c:1652  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1604,14 +1609,13 @@ yyreduce:
   /* Now 'shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
-
-  yyn = yyr1[yyn];
-
-  yystate = yypgoto[yyn - YYNTOKENS] + *yyssp;
-  if (0 <= yystate && yystate <= YYLAST && yycheck[yystate] == *yyssp)
-    yystate = yytable[yystate];
-  else
-    yystate = yydefgoto[yyn - YYNTOKENS];
+  {
+    const int yylhs = yyr1[yyn] - YYNTOKENS;
+    const int yyi = yypgoto[yylhs] + *yyssp;
+    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
+               ? yytable[yyi]
+               : yydefgoto[yylhs]);
+  }
 
   goto yynewstate;
 
@@ -1694,12 +1698,10 @@ yyerrlab:
 | yyerrorlab -- error raised explicitly by YYERROR.  |
 `---------------------------------------------------*/
 yyerrorlab:
-
-  /* Pacify compilers like GCC when the user code never invokes
-     YYERROR and the label yyerrorlab therefore never appears in user
-     code.  */
-  if (/*CONSTCOND*/ 0)
-     goto yyerrorlab;
+  /* Pacify compilers when the user code never invokes YYERROR and the
+     label yyerrorlab therefore never appears in user code.  */
+  if (0)
+    YYERROR;
 
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
@@ -1761,6 +1763,7 @@ yyacceptlab:
   yyresult = 0;
   goto yyreturn;
 
+
 /*-----------------------------------.
 | yyabortlab -- YYABORT comes here.  |
 `-----------------------------------*/
@@ -1768,6 +1771,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
+
 #if !defined yyoverflow || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
@@ -1778,6 +1782,10 @@ yyexhaustedlab:
   /* Fall through.  */
 #endif
 
+
+/*-----------------------------------------------------.
+| yyreturn -- parsing is finished, return the result.  |
+`-----------------------------------------------------*/
 yyreturn:
   if (yychar != YYEMPTY)
     {
@@ -1807,7 +1815,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 199 "parser.y" /* yacc.c:1906  */
+#line 199 "parser.y" /* yacc.c:1918  */
 
 /*************************************************************************
    Main and more
