@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2015-2020 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -25,6 +25,7 @@
 #include <cctype>
 #include <cstring>
 #include <map>
+#include <numeric>
 #include <spot/twa/acc.hh>
 #include "spot/priv/bddalloc.hh"
 #include <spot/misc/minato.hh>
@@ -151,7 +152,7 @@ namespace spot
               {
                 if (!top)
                   // Avoid extra parentheses if there is only one set
-                  top = code[pos - 1].mark.count() == 1;
+                  top = code[pos - 1].mark.is_singleton();
                 unsigned level = 0;
                 const char* and_ = "";
                 const char* and_next_ = []() {
@@ -206,7 +207,7 @@ namespace spot
               {
                 if (!top)
                   // Avoid extra parentheses if there is only one set
-                  top = code[pos - 1].mark.count() == 1;
+                  top = code[pos - 1].mark.is_singleton();
                 unsigned level = 0;
                 const char* or_ = "";
                 if (!top)
@@ -490,7 +491,7 @@ namespace spot
             }
           if (o1 != acc_cond::acc_op::Fin
               || o2 != acc_cond::acc_op::Inf
-              || m1.count() != 1
+              || !m1.is_singleton()
               || m2 != (m1 << 1))
             return false;
           seen_fin |= m1;
@@ -518,7 +519,7 @@ namespace spot
           assert(code.size() == 2);
 
           auto m = code[0].mark;
-          if (mainop == singleop && m.count() != 1)
+          if (mainop == singleop && !m.is_singleton())
             return false;
 
           acc_cond::mark_t fin = {};
@@ -556,7 +557,7 @@ namespace spot
               acc_cond::mark_t fin = {};
               acc_cond::mark_t inf = {};
 
-              if (op == singleop && m.count() != 1)
+              if (op == singleop && !m.is_singleton())
                 {
                   pairs.clear();
                   return false;
@@ -594,8 +595,8 @@ namespace spot
                 }
               if (o1 != acc_cond::acc_op::Fin
                   || o2 != acc_cond::acc_op::Inf
-                  || m1.count() != 1
-                  || m2.count() != 1)
+                  || !m1.is_singleton()
+                  || !m2.is_singleton())
                 {
                   pairs.clear();
                   return false;
@@ -681,14 +682,20 @@ namespace spot
 
     // generalized Rabin should start with Or, unless
     // it has a single pair.
+    bool singlepair = false;
     {
       auto topop = code_.back().sub.op;
       if (topop == acc_op::And)
-        // Probably single-pair generalized-Rabin.  Shift the source
-        // by one position as if we had an invisible Or in front.
-        ++s;
+        {
+          // Probably single-pair generalized-Rabin.  Shift the source
+          // by one position as if we had an invisible Or in front.
+          ++s;
+          singlepair = true;
+        }
       else if (topop != acc_op::Or)
-        return false;
+        {
+          return false;
+        }
     }
 
     // Each pair is the position of a Fin followed
@@ -716,7 +723,7 @@ namespace spot
 
             if (o1 != acc_cond::acc_op::Fin
                 || o2 != acc_cond::acc_op::Inf
-                || m1.count() != 1)
+                || !m1.is_singleton())
               return false;
 
             unsigned i = m2.count();
@@ -752,7 +759,8 @@ namespace spot
       }
     for (auto i: p)
       pairs.emplace_back(i.second);
-    return (!(seen_fin & seen_inf)
+    return ((!singlepair || pairs.size() == 1)
+            && !(seen_fin & seen_inf)
             && (seen_fin | seen_inf) == all_sets());
   }
 
@@ -776,14 +784,20 @@ namespace spot
 
     // generalized Streett should start with And, unless
     // it has a single pair.
+    bool singlepair = false;
     {
       auto topop = code_.back().sub.op;
       if (topop == acc_op::Or)
-        // Probably single-pair generalized-Streett.  Shift the source
-        // by one position as if we had an invisible And in front.
-        ++s;
+        {
+          // Probably single-pair generalized-Streett.  Shift the source
+          // by one position as if we had an invisible And in front.
+          ++s;
+          singlepair = true;
+        }
       else if (topop != acc_op::And)
-        return false;
+        {
+          return false;
+        }
     }
 
 
@@ -812,7 +826,7 @@ namespace spot
 
             if (o1 != acc_cond::acc_op::Inf
                 || o2 != acc_cond::acc_op::Fin
-                || m1.count() != 1)
+                || !m1.is_singleton())
               return false;
 
             unsigned i = m2.count();
@@ -847,7 +861,8 @@ namespace spot
       }
     for (auto i: p)
       pairs.emplace_back(i.second);
-    return (!(seen_inf & seen_fin)
+    return ((!singlepair || pairs.size() == 1)
+            && !(seen_inf & seen_fin)
             && (seen_inf | seen_fin) == all_sets());
   }
 
@@ -1045,6 +1060,81 @@ namespace spot
     return res;
   }
 
+  namespace
+  {
+    bool
+    has_parity_prefix_aux(acc_cond original,
+                          acc_cond &new_cond,
+                          std::vector<unsigned> &colors,
+                          std::vector<acc_cond::acc_code> elements,
+                          acc_cond::acc_op op)
+    {
+      if (elements.size() > 2)
+        {
+          new_cond = original;
+          return false;
+        }
+      if (elements.size() == 2)
+        {
+          unsigned pos = (elements[1].back().sub.op == op
+                          && elements[1][0].mark.is_singleton());
+          if (!(elements[0].back().sub.op == op || pos))
+            {
+              new_cond = original;
+              return false;
+            }
+          if ((elements[1 - pos].used_sets() & elements[pos][0].mark))
+            {
+              new_cond = original;
+              return false;
+            }
+          if (!elements[pos][0].mark.is_singleton())
+            {
+              return false;
+            }
+          colors.push_back(elements[pos][0].mark.min_set() - 1);
+          elements[1 - pos].has_parity_prefix(new_cond, colors);
+          return true;
+        }
+      return false;
+    }
+  }
+
+  bool
+  acc_cond::acc_code::has_parity_prefix(acc_cond &new_cond,
+                                        std::vector<unsigned> &colors) const
+  {
+    auto disj = top_disjuncts();
+    if (!(has_parity_prefix_aux((*this), new_cond, colors,
+            disj, acc_cond::acc_op::Inf) ||
+          has_parity_prefix_aux((*this), new_cond, colors,
+            top_conjuncts(), acc_cond::acc_op::Fin)))
+      new_cond = acc_cond((*this));
+    return disj.size() == 2;
+  }
+
+  bool
+  acc_cond::has_parity_prefix(acc_cond& new_cond,
+                              std::vector<unsigned>& colors) const
+  {
+    return code_.has_parity_prefix(new_cond, colors);
+  }
+
+  bool
+  acc_cond::is_parity_max_equiv(std::vector<int>&permut, bool even) const
+  {
+    if (code_.used_once_sets() != code_.used_sets())
+      return false;
+    bool result = code_.is_parity_max_equiv(permut, 0, even);
+    int max_value = *std::max_element(std::begin(permut), std::end(permut));
+    for (unsigned i = 0; i < permut.size(); ++i)
+      if (permut[i] != -1)
+        permut[i] = max_value - permut[i];
+      else
+        permut[i] = i;
+    return result;
+  }
+
   bool acc_cond::is_parity(bool& max, bool& odd, bool equiv) const
   {
     unsigned sets = num_;
@@ -1236,6 +1326,111 @@ namespace spot
     return rescode;
   }
 
+  bool
+  acc_cond::acc_code::is_parity_max_equiv(std::vector<int>& permut,
+                                          unsigned new_color,
+                                          bool even) const
+  {
+    auto conj = top_conjuncts();
+    auto disj = top_disjuncts();
+    if (conj.size() == 1)
+      {
+        if (disj.size() == 1)
+          {
+            acc_cond::acc_code elem = conj[0];
+            if ((even && elem.back().sub.op == acc_cond::acc_op::Inf)
+                || (!even && elem.back().sub.op == acc_cond::acc_op::Fin))
+              {
+                for (auto color : disj[0][0].mark.sets())
+                  {
+                    if (permut[color] != -1
+                        && ((unsigned) permut[color]) != new_color)
+                      return false;
+                    permut[color] = new_color;
+                  }
+                return true;
+              }
+            return false;
+          }
+        else
+          {
+            std::sort(disj.begin(), disj.end(),
+                      [](acc_code c1, acc_code c2)
+                      {
+                        return (c1 != c2) &&
+                          c1.back().sub.op == acc_cond::acc_op::Inf;
+                      });
+            unsigned i = 0;
+            for (; i < disj.size() - 1; ++i)
+              {
+                if (disj[i].back().sub.op != acc_cond::acc_op::Inf
+                    || !disj[i][0].mark.is_singleton())
+                  return false;
+                for (auto color : disj[i][0].mark.sets())
+                  {
+                    if (permut[color] != -1
+                        && ((unsigned) permut[color]) != new_color)
+                      return false;
+                    permut[color] = new_color;
+                  }
+              }
+            if (disj[i].back().sub.op == acc_cond::acc_op::Inf)
+              {
+                if (!even || !disj[i][0].mark.is_singleton())
+                  return false;
+                for (auto color : disj[i][0].mark.sets())
+                  {
+                    if (permut[color] != -1
+                        && ((unsigned) permut[color]) != new_color)
+                      return false;
+                    permut[color] = new_color;
+                  }
+                return true;
+              }
+            return disj[i].is_parity_max_equiv(permut, new_color + 1, even);
+          }
+      }
+    else
+      {
+        std::sort(conj.begin(), conj.end(),
+                  [](acc_code c1, acc_code c2)
+                  {
+                    return (c1 != c2)
+                      && c1.back().sub.op == acc_cond::acc_op::Fin;
+                  });
+        unsigned i = 0;
+        for (; i < conj.size() - 1; i++)
+          {
+            if (conj[i].back().sub.op != acc_cond::acc_op::Fin
+                || !conj[i][0].mark.is_singleton())
+              return false;
+            for (auto color : conj[i][0].mark.sets())
+              {
+                if (permut[color] != -1 && permut[color != new_color])
+                  return false;
+                permut[color] = new_color;
+              }
+          }
+        if (conj[i].back().sub.op == acc_cond::acc_op::Fin)
+          {
+            if (even)
+              return 0;
+            if (!conj[i][0].mark.is_singleton())
+              return false;
+            for (auto color : conj[i][0].mark.sets())
+              {
+                if (permut[color] != -1 && permut[color != new_color])
+                  return false;
+                permut[color] = new_color;
+              }
+            return true;
+          }
+
+        return conj[i].is_parity_max_equiv(permut, new_color + 1, even);
+      }
+  }
+
+
   namespace
   {
     template<typename Fun>
@@ -1493,7 +1688,7 @@ namespace spot
             break;
           case acc_cond::acc_op::Fin:
           case acc_cond::acc_op::FinNeg:
-            if (pos[-1].mark.count() > 1 && pos > and_scope)
+            if (pos[-1].mark.has_many() && pos > and_scope)
               return false;
             SPOT_FALLTHROUGH;
           case acc_cond::acc_op::Inf:
@@ -1526,7 +1721,7 @@ namespace spot
             break;
           case acc_cond::acc_op::Inf:
           case acc_cond::acc_op::InfNeg:
-            if (pos[-1].mark.count() > 1 && pos > or_scope)
+            if (pos[-1].mark.has_many() && pos > or_scope)
               return false;
             SPOT_FALLTHROUGH;
           case acc_cond::acc_op::Fin:
@@ -2340,6 +2535,7 @@ namespace spot
 
     std::swap(c, *this);
   }
+
   int acc_cond::acc_code::fin_one() const
   {
     if (empty() || is_f())
@@ -2366,6 +2562,154 @@ namespace spot
     return -1;
   }
 
+  namespace
+  {
+    bool
+    find_unit_clause(acc_cond::acc_code code, bool& conj, bool& fin,
+                     acc_cond::mark_t& res)
+    {
+      res = {};
+      bool found_one = false;
+      conj = false;
+      fin = false;
+      if (code.empty() || code.is_f())
+        return false;
+      acc_cond::mark_t candidates = ~code.used_once_sets();
+      if (!candidates)
+        return false;
+      const acc_cond::acc_word* pos = &code.back();
+      conj = (pos->sub.op == acc_cond::acc_op::And);
+      do
+        {
+          switch (pos->sub.op)
+            {
+            case acc_cond::acc_op::And:
+              if (!conj)
+                pos -= pos->sub.size + 1;
+              else
+                --pos;
+              break;
+            case acc_cond::acc_op::Or:
+              if (conj)
+                pos -= pos->sub.size + 1;
+              else
+                --pos;
+              break;
+            case acc_cond::acc_op::Inf:
+            case acc_cond::acc_op::InfNeg:
+            case acc_cond::acc_op::FinNeg:
+              if (!fin)
+                {
+                  auto m = pos[-1].mark & candidates;
+                  if (m && (conj || pos[-1].mark.is_singleton()))
+                    {
+                      found_one = true;
+                      res |= m;
+                    }
+                }
+              pos -= 2;
+              break;
+            case acc_cond::acc_op::Fin:
+              if (!found_one || fin)
+                {
+                  auto m = pos[-1].mark & candidates;
+                  if (m && (!conj || pos[-1].mark.is_singleton()))
+                    {
+                      found_one = true;
+                      fin = true;
+                      res |= m;
+                    }
+                }
+              pos -= 2;
+              break;
+            }
+        }
+      while (pos >= &code.front());
+      return !!res;
+    }
+  }
+
+  acc_cond::acc_code
+  acc_cond::acc_code::unit_propagation()
+  {
+    bool fin, conj;
+    acc_cond::mark_t mark;
+    acc_code result = (*this);
+    acc_code code;
+    if (result.size() > 1 &&
+        (result.back().sub.op == acc_cond::acc_op::And
+        || result.back().sub.op == acc_cond::acc_op::Or))
+    {
+      while (find_unit_clause(result, conj, fin, mark))
+      {
+        acc_code init_code;
+
+        if (fin)
+        {
+          if (conj)
+          {
+            init_code = acc_code::t();
+            for (unsigned col : mark.sets())
+              init_code &= acc_code::fin({col});
+          } else
+          {
+            init_code = acc_code::f();
+            for (unsigned col : mark.sets())
+              init_code |= acc_code::fin({col});
+          }
+        }
+        else
+        {
+          if (conj)
+          {
+            init_code = acc_code::t();
+            for (unsigned col : mark.sets())
+              init_code &= acc_code::inf({col});
+          }
+          else
+          {
+            init_code = acc_code::f();
+            for (unsigned col : mark.sets())
+              init_code |= acc_code::inf({col});
+          }
+        }
+        if (conj)
+          result = init_code & result.remove(mark, fin == conj);
+        else
+          result = init_code | result.remove(mark, fin == conj);
+      }
+
+      if (result.is_t())
+        return result;
+      auto pos = &result.back();
+      auto fo = pos->sub.op;
+      bool is_and = (fo == acc_cond::acc_op::And);
+      std::vector<acc_cond::acc_code> propagated;
+      acc_cond::acc_code res;
+      if (is_and)
+        res = acc_cond::acc_code::t();
+      else
+        res = acc_cond::acc_code::f();
+      if (is_and || fo == acc_cond::acc_op::Or)
+      {
+        --pos;
+        while (pos >= &result.front())
+        {
+          propagated.push_back(acc_code(pos).unit_propagation());
+          pos -= pos->sub.size + 1;
+        }
+        result = std::accumulate(propagated.rbegin(), propagated.rend(), res,
+        [&](acc_cond::acc_code c1, acc_cond::acc_code c2)
+        {
+          if (is_and)
+            return c1 & c2;
+          return c1 | c2;
+        });
+      }
+    }
+    return result;
+  }
+
   acc_cond::mark_t acc_cond::acc_code::fin_unit() const
   {
     mark_t res = {};
@@ -2389,7 +2733,7 @@ namespace spot
             break;
           case acc_cond::acc_op::Fin:
             auto m = pos[-1].mark;
-            if (m.count() == 1)
+            if (m.is_singleton())
               res |= m;
             pos -= 2;
             break;
@@ -2398,4 +2742,34 @@ namespace spot
     while (pos >= &front());
     return res;
   }
+
+  acc_cond::mark_t acc_cond::acc_code::used_once_sets() const
+  {
+    mark_t seen = {};
+    mark_t dups = {};
+    if (empty())
+      return {};
+    const acc_cond::acc_word* pos = &back();
+    do
+      {
+        switch (pos->sub.op)
+          {
+          case acc_cond::acc_op::And:
+          case acc_cond::acc_op::Or:
+            --pos;
+            break;
+          case acc_cond::acc_op::Inf:
+          case acc_cond::acc_op::InfNeg:
+          case acc_cond::acc_op::FinNeg:
+          case acc_cond::acc_op::Fin:
+            auto m = pos[-1].mark;
+            pos -= 2;
+            dups |= m & seen;
+            seen |= m;
+            break;
+          }
+      }
+    while (pos >= &front());
+    return seen - dups;
+  }
 }
