@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2014-2020 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -31,6 +31,15 @@ using namespace std::string_literals;
 namespace spot
 {
 
+  void
+  twa_graph::apply_permutation(std::vector<unsigned> permut)
+  {
+    for (auto& e : edges())
+    {
+      e.acc.apply_permutation(permut);
+    }
+  }
+
   std::string twa_graph::format_state(unsigned n) const
   {
     if (is_univ_dest(n))
@@ -148,7 +157,8 @@ namespace spot
                    });
 
     auto& trans = this->edge_vector();
-    unsigned tend = trans.size();
+    unsigned orig_size = trans.size();
+    unsigned tend = orig_size;
     unsigned out = 0;
     unsigned in = 1;
     // Skip any leading false edge.
@@ -233,6 +243,13 @@ namespace spot
       }
 
     g_.chain_edges_();
+
+    // Did we actually reduce the number of edges?
+    if (trans.size() != orig_size)
+      // Merging some edges may turn a non-deterministic automaton
+      // into a deterministic one.
+      if (prop_universal().is_false())
+        prop_universal(trival::maybe());
   }
 
   void twa_graph::merge_states()
@@ -947,7 +964,7 @@ namespace spot
       // States already seen.
       state_map<unsigned> seen;
       // States to process
-      std::deque<state_map<unsigned>::const_iterator> todo;
+      std::deque<state_map<unsigned>::value_type> todo;
 
       auto new_state = [&](const state* s) -> unsigned
         {
@@ -955,7 +972,7 @@ namespace spot
           if (p.second)
             {
               p.first->second = out->new_state();
-              todo.emplace_back(p.first);
+              todo.emplace_back(*p.first);
               if (names)
                 names->emplace_back(aut->format_state(s));
               if (ohstates)
@@ -977,7 +994,7 @@ namespace spot
         {
           const state* src1;
           unsigned src2;
-          std::tie(src1, src2) = *todo.front();
+          std::tie(src1, src2) = todo.front();
           todo.pop_front();
           for (auto* t: aut->succ(src1))
             {
