@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2017 Laboratoire de Recherche et Développement
+// Copyright (C) 2013-2018 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/twaalgos/complete.hh>
 
 namespace spot
@@ -152,7 +153,7 @@ namespace spot
                 // in case the automaton uses state-based acceptance, propagate
                 // the acceptance of the first edge to the one we add.
                 if (!aut->prop_state_acc().is_true() && i != sink)
-                  acc = 0U;
+                  acc = {};
 
                 aut->new_edge(i, sink, missingcond, acc);
               }
