@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2018 Laboratoire de Recherche
+// Copyright (C) 2012-2020 Laboratoire de Recherche
 // et Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -29,6 +29,7 @@
 #include <memory>
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/twa/bddprint.hh>
+#include <spot/twaalgos/isdet.hh>
 
 //#define DEGEN_DEBUG
 
@@ -122,7 +123,7 @@ namespace spot
         unsigned n = a->num_states();
         acc_cond::mark_t all = a_->acc().all_sets();
         // slot 2 will hold acceptance mark that are common to the
-        // incoming transitions of each state.  For know with all
+        // incoming transitions of each state.  For now with all
         // marks if there is some incoming edge.  The next loop will
         // constrain this value.
         for (auto& e: a_->edges())
@@ -176,36 +177,59 @@ namespace spot
     // Order of accepting sets (for one SCC)
     class acc_order final
     {
+      std::vector<std::pair<acc_cond::mark_t, unsigned>> found_;
       std::vector<unsigned> order_;
-      acc_cond::mark_t found_;
 
     public:
-      unsigned
-      next_level(int slevel, acc_cond::mark_t set, bool skip_levels)
+      acc_order(acc_cond::mark_t all)
       {
-        // Update the order with any new set we discover
-        if (auto newsets = set - found_)
-          {
-            newsets.fill(std::back_inserter(order_));
-            found_ |= newsets;
-          }
+        unsigned count = all.count();
+        found_.emplace_back(all, count);
+        order_.insert(order_.begin(), count, 0);
+      }
 
-        unsigned next = slevel;
-        while (next < order_.size() && set.has(order_[next]))
+      unsigned
+      next_level(unsigned slevel, acc_cond::mark_t set)
+      {
+        unsigned last = order_.size();
+        if (last == 0)
+          return slevel;
+        unsigned index = order_[slevel];
+
+        while (slevel < last
+               && (found_[index].first & set) == found_[index].first)
+          slevel += found_[index++].second;
+        if (slevel == last)
+          return slevel;
+        if (acc_cond::mark_t match = (found_[index].first & set))
           {
-            ++next;
-            if (!skip_levels)
-              break;
+            unsigned matchsize = match.count();
+            found_[index].first -= match;
+            found_[index].second -= matchsize;
+            found_.emplace(found_.begin() + index, match, matchsize);
+            slevel += matchsize;
+            // update order_
+            unsigned opos = 0;
+            unsigned fpos = 0;
+            for (auto& p: found_)
+              {
+                for (unsigned n = 0; n < p.second; ++n)
+                  order_[opos++] = fpos;
+                ++fpos;
+              }
           }
-        return next;
+        return slevel;
       }
 
       void
       print(int scc)
       {
         std::cout << "Order_" << scc << ":\t";
+        for (auto i: found_)
+          std::cout << i.first << ' ';
+        std::cout << " // ";
         for (auto i: order_)
-          std::cout << i << ", ";
+          std::cout << i << ' ';
         std::cout << '\n';
       }
     };
@@ -213,30 +237,92 @@ namespace spot
     // Accepting order for each SCC
     class scc_orders final
     {
-      std::map<int, acc_order> orders_;
-      bool skip_levels_;
+      std::vector<acc_order> orders_;
 
     public:
-      scc_orders(bool skip_levels):
-        skip_levels_(skip_levels)
+      scc_orders(acc_cond::mark_t all, unsigned scc_count)
+        : orders_(scc_count, acc_order(all))
       {
       }
 
       unsigned
       next_level(int scc, int slevel, acc_cond::mark_t set)
       {
-        return orders_[scc].next_level(slevel, set, skip_levels_);
+        return orders_[scc].next_level(slevel, set);
       }
 
       void
       print()
       {
-        std::map<int, acc_order>::iterator i;
-        for (i = orders_.begin(); i != orders_.end(); i++)
-          i->second.print(i->first);
+        unsigned sz = orders_.size();
+        for (unsigned i = 0; i < sz; ++i)
+          orders_[i].print(i);
       }
     };
 
+
+    namespace
+    {
+      static void
+      keep_bottommost_copies(twa_graph_ptr& res,
+                             scc_info& si_orig,
+                             std::vector<unsigned>* orig_states,
+                             bool force_purge = false)
+      {
+        unsigned res_ns = res->num_states();
+        auto a = si_orig.get_aut();
+        if (res_ns <= a->num_states())
+          return;
+
+        scc_info si_res(res, scc_info_options::TRACK_STATES);
+        unsigned res_scc_count = si_res.scc_count();
+        if (res_scc_count <= si_orig.scc_count())
+          {
+            if (force_purge)
+              res->purge_unreachable_states();
+            return;
+          }
+
+        // If we reach this place, we have more SCCs in the output than
+        // in the input.  This means that we have created some redundant
+        // SCCs.  Often, these are trivial SCCs created in front of
+        // their larger sisters, because we did not pick the correct
+        // level when entering the SCC for the first time, and the level
+        // we picked has not been seen again when exploring the SCC.
+        // But it could also be the case that by entering the SCC in two
+        // different ways, we create two clones of the SCC (I haven't
+        // encountered any such case, but I do not want to rule it out
+        // in the code below).
+        //
+        // Now we will iterate over the SCCs in topological order to
+        // remember the "bottommost" SCCs that contain each original
+        // state.  If an original state is duplicated in a higher SCC,
+        // it can be shunted away.  Amen.
+        unsigned maxorig = *std::max_element(orig_states->begin(),
+                                             orig_states->end());
+        std::vector<unsigned>
+          bottommost_occurence(maxorig + 1);
+        {
+          unsigned n = res_scc_count;
+          do
+            for (unsigned s: si_res.states_of(--n))
+              bottommost_occurence[(*orig_states)[s]] = s;
+          while (n);
+        }
+        std::vector<unsigned> retarget(res_ns);
+        for (unsigned n = 0; n < res_ns; ++n)
+          {
+            unsigned other = bottommost_occurence[(*orig_states)[n]];
+            retarget[n] =
+              (si_res.scc_of(n) != si_res.scc_of(other)) ? other : n;
+          }
+        for (auto& e: res->edges())
+          e.dst = retarget[e.dst];
+        res->set_init_state(retarget[res->get_init_state_number()]);
+        res->purge_unreachable_states();
+      }
+    }
+
     template<bool want_sba>
     twa_graph_ptr
     degeneralize_aux(const const_twa_graph_ptr& a, bool use_z_lvl,
@@ -267,6 +353,9 @@ namespace spot
       // Preserve determinism, weakness, and stutter-invariance
       res->prop_copy(a, { false, true, true, true, true, true });
 
+      auto old_orig_states =
+        a->get_named_prop<std::vector<unsigned>>("original-states");
+
       auto orig_states = new std::vector<unsigned>();
       auto levels = new std::vector<unsigned>();
       orig_states->reserve(a->num_states()); // likely more are needed.
@@ -292,9 +381,6 @@ namespace spot
           order.emplace_back(i - 1);
       }
 
-      // Initialize scc_orders
-      scc_orders orders(skip_levels);
-
       // and vice-versa.
       ds2num_map ds2num;
 
@@ -314,6 +400,11 @@ namespace spot
         ? std::make_unique<scc_info>(a, scc_info_options::NONE)
         : nullptr;
 
+      // Initialize scc_orders
+      std::unique_ptr<scc_orders> orders = use_cust_acc_orders
+        ? std::make_unique<scc_orders>(a->acc().all_sets(), m->scc_count())
+        : nullptr;
+
       // Cache for common outgoing/incoming acceptances.
       inout_acc inout(a, m.get());
 
@@ -334,7 +425,7 @@ namespace spot
         {
           auto set = inout.common_inout_acc(s.first);
           if (SPOT_UNLIKELY(use_cust_acc_orders))
-            s.second = orders.next_level(m->initial(), s.second, set);
+            s.second = orders->next_level(m->initial(), s.second, set);
           else
             while (s.second < order.size() && set.has(order[s.second]))
               {
@@ -373,7 +464,10 @@ namespace spot
             todo.emplace_back(ds);
 
           assert(ns == orig_states->size());
-          orig_states->emplace_back(ds.first);
+          unsigned orig = ds.first;
+          if (old_orig_states)
+            orig = (*old_orig_states)[orig];
+          orig_states->emplace_back(orig);
           levels->emplace_back(ds.second);
 
           // Level cache stores one encountered level for each state
@@ -533,7 +627,7 @@ namespace spot
                       // for this scc
                       if (use_cust_acc_orders)
                         {
-                          d.second = orders.next_level(scc, next, acc);
+                          d.second = orders->next_level(scc, next, acc);
                         }
                       // Else compute level according the global acc order
                       else
@@ -572,7 +666,7 @@ namespace spot
 
               // In case we are building a TBA is_acc has to be
               // set differently for each edge, and
-              // we do not need to stay use final level.
+              // we do not need to stay on final level.
               if (!want_sba)
                 {
                   is_acc = d.second == order.size();
@@ -580,11 +674,11 @@ namespace spot
                     {
                       d.second = 0; // Make it go to the first level.
                       // Skip as many levels as possible.
-                      if (!a->acc().accepting(acc) && !skip_levels)
+                      if (!a->acc().accepting(acc) && skip_levels)
                         {
                           if (use_cust_acc_orders)
                             {
-                              d.second = orders.next_level(scc, d.second, acc);
+                              d.second = orders->next_level(scc, d.second, acc);
                             }
                           else
                             {
@@ -614,51 +708,11 @@ namespace spot
       for (auto i: order)
         std::cout << i << ", ";
       std::cout << '\n';
-      orders.print();
+      orders->print();
 #endif
       res->merge_edges();
-
-      unsigned res_ns = res->num_states();
-      if (!remove_extra_scc || res_ns <= a->num_states())
-        return res;
-
-      scc_info si_res(res, scc_info_options::TRACK_STATES);
-      unsigned res_scc_count = si_res.scc_count();
-      if (res_scc_count <= m->scc_count())
-        return res;
-
-      // If we reach this place, we have more SCCs in the output than
-      // in the input.  This means that we have created some redundant
-      // SCCs.  Often, these are trivial SCCs created in front of
-      // their larger sisters, because we did not pick the correct
-      // level when entering the SCC for the first time, and the level
-      // we picked has not been seen again when exploring the SCC.
-      // But it could also be the case that by entering the SCC in two
-      // different ways, we create two clones of the SCC (I haven't
-      // encountered any such case, but I do not want to rule it out
-      // in the code below).
-      //
-      // Now we will iterate over the SCCs in topological order to
-      // remember the "bottomost" SCCs that contain each original
-      // state.  If an original state is duplicated in a higher SCC,
-      // it can be shunted away.  Amen.
-      std::vector<unsigned> bottomost_occurence(a->num_states());
-      {
-        unsigned n = res_scc_count;
-        do
-          for (unsigned s: si_res.states_of(--n))
-            bottomost_occurence[(*orig_states)[s]] = s;
-        while (n);
-      }
-      std::vector<unsigned> retarget(res_ns);
-      for (unsigned n = 0; n < res_ns; ++n)
-        {
-          unsigned other = bottomost_occurence[(*orig_states)[n]];
-          retarget[n] = (si_res.scc_of(n) != si_res.scc_of(other)) ? other : n;
-        }
-      for (auto& e: res->edges())
-        e.dst = retarget[e.dst];
-      res->purge_unreachable_states();
+      if (remove_extra_scc)
+        keep_bottommost_copies(res, *(m.get()), orig_states);
       return res;
     }
   }
@@ -694,4 +748,415 @@ namespace spot
                                    use_lvl_cache, skip_levels, ignaccsl,
                                    remove_extra_scc);
   }
+
+  namespace
+  {
+    static acc_cond::mark_t
+    to_strip(const acc_cond::acc_code& code, acc_cond::mark_t todegen)
+    {
+      if (code.empty())
+        return todegen;
+
+      acc_cond::mark_t tostrip = todegen;
+      unsigned pos = code.size();
+      do
+        {
+          switch (code[pos - 1].sub.op)
+            {
+            case acc_cond::acc_op::And:
+            case acc_cond::acc_op::Or:
+              --pos;
+              break;
+            case acc_cond::acc_op::Fin:
+            case acc_cond::acc_op::FinNeg:
+            case acc_cond::acc_op::Inf:
+            case acc_cond::acc_op::InfNeg:
+              {
+                pos -= 2;
+                acc_cond::mark_t m = code[pos].mark;
+                if (todegen.subset(m))
+                  m -= todegen;
+                tostrip -= m;
+                break;
+              }
+            }
+        }
+      while (pos > 0);
+      return tostrip;
+    }
+
+    static bool
+    update_acc_for_partial_degen(acc_cond::acc_code& code,
+                                 acc_cond::mark_t todegen,
+                                 acc_cond::mark_t tostrip,
+                                 acc_cond::mark_t accmark)
+    {
+      if (!todegen || code.empty())
+        {
+          code &= acc_cond::acc_code::inf(accmark);
+          return true;
+        }
+
+      bool updated = false;
+      unsigned pos = code.size();
+      do
+        {
+          switch (code[pos - 1].sub.op)
+            {
+            case acc_cond::acc_op::And:
+            case acc_cond::acc_op::Or:
+              --pos;
+              break;
+            case acc_cond::acc_op::Fin:
+            case acc_cond::acc_op::FinNeg:
+            case acc_cond::acc_op::Inf:
+            case acc_cond::acc_op::InfNeg:
+              {
+                pos -= 2;
+                acc_cond::mark_t m = code[pos].mark;
+                if (todegen.subset(m))
+                  {
+                    m -= todegen;
+                    code[pos].mark = m.strip(tostrip) | accmark;
+                    updated = true;
+                  }
+                else
+                  {
+                    code[pos].mark = m.strip(tostrip);
+                  }
+                break;
+              }
+            }
+        }
+      while (pos > 0);
+      return updated;
+    }
+
+    [[noreturn]] static void
+    report_invalid_partial_degen_arg(acc_cond::mark_t todegen,
+                                     acc_cond::acc_code& cond)
+    {
+      std::ostringstream err;
+      err << "partial_degeneralize(): " << todegen
+          << " does not match any degeneralizable subformula of "
+          << cond << '.';
+      throw std::runtime_error(err.str());
+    }
+  }
+
+  acc_cond::mark_t
+  is_partially_degeneralizable(const const_twa_graph_ptr& aut,
+                               bool allow_inf, bool allow_fin,
+                               std::vector<acc_cond::mark_t> forbid)
+  {
+    auto& code = aut->get_acceptance();
+
+    if (code.empty())
+      return {};
+
+    acc_cond::mark_t res = {};
+    unsigned res_sz = -1U;
+    auto update = [&](const acc_cond::mark_t& m)
+    {
+      unsigned sz = m.count();
+      if (sz > 1 && sz < res_sz)
+        {
+          res_sz = sz;
+          res = m;
+        }
+      // If we have found a pair to degeneralize, we
+      // won't find
+      return res_sz == 2;
+    };
+
+    unsigned pos = code.size();
+    do
+      {
+        switch (code[pos - 1].sub.op)
+          {
+          case acc_cond::acc_op::And:
+          case acc_cond::acc_op::Or:
+            --pos;
+            break;
+          case acc_cond::acc_op::Fin:
+          case acc_cond::acc_op::FinNeg:
+            pos -= 2;
+            if (allow_fin)
+            {
+              auto m = code[pos].mark;
+              if (!std::count(forbid.begin(), forbid.end(), m) && update(m))
+                return res;
+            }
+            break;
+          case acc_cond::acc_op::Inf:
+          case acc_cond::acc_op::InfNeg:
+            pos -= 2;
+            if (allow_inf)
+            {
+              auto m = code[pos].mark;
+              if (!std::count(forbid.begin(), forbid.end(), m) && update(m))
+                return res;
+            }
+            break;
+          }
+      }
+    while (pos > 0);
+    return res;
+  }
+
+
+  twa_graph_ptr
+  partial_degeneralize(const const_twa_graph_ptr& a,
+                       acc_cond::mark_t todegen)
+
+  {
+    auto res = make_twa_graph(a->get_dict());
+    res->copy_ap_of(a);
+    acc_cond::acc_code acc = a->get_acceptance();
+
+    acc_cond::mark_t tostrip = to_strip(acc, todegen);
+    acc_cond::mark_t keep = a->acc().all_sets() - tostrip;
+    acc_cond::mark_t degenmark = {keep.count()};
+
+    if (!update_acc_for_partial_degen(acc, todegen, tostrip, degenmark))
+      report_invalid_partial_degen_arg(todegen, acc);
+    res->set_acceptance(acc);
+
+    // auto* names = new std::vector<std::string>;
+    // res->set_named_prop("state-names", names);
+    auto orig_states = new std::vector<unsigned>();
+    auto levels = new std::vector<unsigned>();
+    unsigned ns = a->num_states();
+    orig_states->reserve(ns); // likely more are needed.
+    levels->reserve(a->num_states());
+    res->set_named_prop("original-states", orig_states);
+    res->set_named_prop("degen-levels", levels);
+
+    scc_info si_orig(a, scc_info_options::NONE);
+
+    auto marks = propagate_marks_vector(a, &si_orig);
+
+    std::vector<unsigned> highest_level(ns, 0);
+    // Compute the marks that are common to all incoming or all
+    // outgoing transitions of each state, ignoring self-loops and
+    // out-of-SCC transitions.  Note that because
+    // propagate_marks_verctor() has been used, the intersection
+    // of all incoming marks is equal to the intersection of all
+    // outgoing marks unless the state has no predecessor or no
+    // successor.  We take the outgoing marks because states without
+    // successor are useless (but states without predecessors are not).
+    std::vector<acc_cond::mark_t> inout(ns, todegen);
+    for (auto& e: a->edges())
+      if (e.src != e.dst && si_orig.scc_of(e.src) == si_orig.scc_of(e.dst))
+        {
+          unsigned idx = a->edge_number(e);
+          inout[e.src] &= marks[idx];
+        }
+
+    scc_orders orders(todegen, si_orig.scc_count());
+    unsigned ordersize = todegen.count();
+
+    // degen_states -> new state numbers
+    ds2num_map ds2num;
+
+    queue_t todo;
+
+    auto new_state = [&](degen_state ds)
+                     {
+                       auto di = ds2num.find(ds);
+                       if (di != ds2num.end())
+                         return di->second;
+
+                       highest_level[ds.first] =
+                         std::max(highest_level[ds.first], ds.second);
+
+                       unsigned ns = res->new_state();
+                       ds2num[ds] = ns;
+                       todo.emplace_back(ds);
+
+                       // std::ostringstream os;
+                       // os << ds.first << ',' << ds.second;
+                       // names->push_back(os.str());
+
+                       unsigned orig = ds.first;
+                       assert(ns == orig_states->size());
+                       orig_states->emplace_back(orig);
+                       levels->emplace_back(ds.second);
+                       return ns;
+                     };
+
+    unsigned init = a->get_init_state_number();
+    degen_state s(init, 0);
+    // The initial level can be anything.  As a heuristic, pretend we
+    // have just seen the acceptance condition common to incoming
+    // edges.
+    s.second = orders.next_level(si_orig.scc_of(init), 0, inout[init]);
+    if (s.second == ordersize)
+      s.second = 0;
+
+    new_state(s);
+
+    // A list of edges are that "all accepting" and whose destination
+    // could be redirected to a higher level.
+    std::vector<unsigned> allaccedges;
+
+    while (!todo.empty())
+      {
+        s = todo.back();
+        todo.pop_back();
+
+        int src = ds2num[s];
+        unsigned slevel = s.second;
+        unsigned orig_src = s.first;
+        unsigned scc_src = si_orig.scc_of(orig_src);
+        for (auto& e: a->out(orig_src))
+          {
+            bool saveedge = false;
+            unsigned nextlvl = slevel;
+            acc_cond::mark_t accepting = {};
+            if (si_orig.scc_of(e.dst) == scc_src)
+              {
+                unsigned idx = a->edge_number(e);
+                acc_cond::mark_t acc = marks[idx] & todegen;
+                nextlvl = orders.next_level(scc_src, nextlvl, acc);
+                if (nextlvl == ordersize)
+                  {
+                    accepting = degenmark;
+                    nextlvl = 0;
+                    if ((acc & todegen) != todegen)
+                      {
+                        nextlvl = orders.next_level(scc_src, nextlvl, acc);
+                      }
+                    else
+                      {
+                        // Because we have seen all sets on this
+                        // transition, we can jump to any level we
+                        // like.  As a heuristic, let's jump to the
+                        // highest existing level.
+                        nextlvl = highest_level[e.dst];
+                        if (nextlvl == 0 // probably a new state ->
+                                         // use inout for now
+                            && inout[e.dst] != todegen)
+                          nextlvl = orders.next_level(scc_src, nextlvl,
+                                                      inout[e.dst]);
+                        // It's possible that we have not yet seen the
+                        // highest level yet, so let's save this edge
+                        // and revisit this issue at the end.
+                        saveedge = true;
+                      }
+                  }
+                accepting |= e.acc.strip(tostrip);
+              }
+            else
+              {
+                nextlvl = 0;
+              }
+
+            degen_state ds_dst(e.dst, nextlvl);
+            unsigned dst = new_state(ds_dst);
+            unsigned idx = res->new_edge(src, dst, e.cond, accepting);
+            if (saveedge)
+              allaccedges.push_back(idx);
+          }
+      }
+    // Raise the destination of the "all-accepting" edges to the
+    // highest existing level.  If we do such a redirection, we need
+    // to force keep_bottommost_copies to purge unreachable_states.
+    bool force_purge = false;
+    auto& ev = res->edge_vector();
+    for (unsigned idx: allaccedges)
+      {
+        unsigned dst = ev[idx].dst;
+        unsigned orig_dst = (*orig_states)[dst];
+        unsigned hl = highest_level[orig_dst];
+        unsigned new_dst = ds2num[degen_state{orig_dst, hl}];
+        if (dst != new_dst)
+          {
+            ev[idx].dst = new_dst;
+            force_purge = true;
+          }
+      }
+    // compose original-states with the any previously existing one.
+    // We do that now, because the above loop uses orig_states to
+    // find the local source, but for the bottommost copy below, it's better
+    // if we compose everything.
+    if (auto old_orig_states =
+        a->get_named_prop<std::vector<unsigned>>("original-states"))
+      for (auto& s: *orig_states)
+        s = (*old_orig_states)[s];
+    //orders.print();
+    res->merge_edges();
+    keep_bottommost_copies(res, si_orig, orig_states, force_purge);
+    return res;
+  }
+
+  twa_graph_ptr
+  partial_degeneralize(twa_graph_ptr a)
+  {
+    while (acc_cond::mark_t m = is_partially_degeneralizable(a))
+      a = partial_degeneralize(a, m);
+    return a;
+  }
+
+  std::vector<acc_cond::mark_t>
+  propagate_marks_vector(const const_twa_graph_ptr& aut,
+                         scc_info* si)
+  {
+    bool own_si = true;
+    if (si)
+      own_si = false;
+    else
+      si = new scc_info(aut);
+
+    unsigned ns = aut->num_states();
+    acc_cond::mark_t allm = aut->acc().all_sets();
+    unsigned es = aut->edge_vector().size();
+    std::vector<acc_cond::mark_t> marks(es, acc_cond::mark_t{});
+    const auto& edges = aut->edge_vector();
+    for (unsigned e = 1; e < es; ++e)
+      marks[e] = edges[e].acc;
+
+    std::vector<acc_cond::mark_t> common_in(ns, allm);
+    std::vector<acc_cond::mark_t> common_out(ns, allm);
+
+    for (;;)
+      {
+        bool changed = false;
+        for (auto& e: aut->edges())
+          if (e.src != e.dst && si->scc_of(e.src) == si->scc_of(e.dst))
+            {
+              unsigned idx = aut->edge_number(e);
+              common_in[e.dst] &= marks[idx];
+              common_out[e.src] &= marks[idx];
+            }
+        for (auto& e: aut->edges())
+          if (e.src != e.dst && si->scc_of(e.src) == si->scc_of(e.dst))
+            {
+              unsigned idx = aut->edge_number(e);
+              auto acc = marks[idx] | common_in[e.src] | common_out[e.dst];
+              if (acc != marks[idx])
+                {
+                  marks[idx] = acc;
+                  changed = true;
+                }
+            }
+        if (!changed)
+          break;
+        std::fill(common_in.begin(), common_in.end(), allm);
+        std::fill(common_out.begin(), common_out.end(), allm);
+      }
+    if (own_si)
+      delete si;
+    return marks;
+  }
+
+  void propagate_marks_here(twa_graph_ptr& aut, scc_info* si)
+  {
+    auto marks = propagate_marks_vector(aut, si);
+    for (auto& e: aut->edges())
+      {
+        unsigned idx = aut->edge_number(e);
+        e.acc = marks[idx];
+      }
+  }
 }
