@@ -38,22 +38,6 @@ namespace spot
 {
   namespace
   {
-    using EdgeMask = std::vector<bool>;
-
-    template< typename Edges, typename Apply >
-    void for_each_edge(const const_twa_graph_ptr& aut,
-                       const Edges& edges,
-                       const EdgeMask& mask,
-                       Apply apply)
-    {
-      for (const auto& e: edges)
-        {
-          unsigned edge_id = aut->edge_number(e);
-          if (mask[edge_id])
-            apply(edge_id);
-        }
-    }
-
     // Transforms automaton from transition based acceptance to state based
     // acceptance.
     void make_state_acc(twa_graph_ptr & aut)
@@ -87,7 +71,6 @@ namespace spot
     bool is_scc_tba_type(const_twa_graph_ptr aut,
                          const scc_info& si,
                          const unsigned scc,
-                         const std::vector<bool>& keep,
                          const rs_pairs_view& aut_pairs,
                          std::vector<bool>& final)
     {
@@ -101,10 +84,8 @@ namespace spot
       auto aut_fin_alone = aut_pairs.fins_alone();
       if ((scc_acc & aut_fin_alone) != aut_fin_alone)
         {
-          for_each_edge(aut, si.edges_of(scc), keep, [&](unsigned e)
-            {
-              final[e] = true;
-            });
+          for (auto& e: si.edges_of(scc))
+            final[aut->edge_number(e)] = true;
           return true;
         }
 
@@ -126,14 +107,14 @@ namespace spot
       // final. Otherwise we do not know: it is possible that there is
       // a non-accepting cycle in the SCC that does not visit Fᵢ.
       std::set<unsigned> unknown;
-      for_each_edge(aut, si.inner_edges_of(scc), keep, [&](unsigned e)
+      for (auto& ed: si.inner_edges_of(scc))
         {
-          const auto& ed = aut->edge_data(e);
+          unsigned e = aut->edge_number(ed);
           if (ed.acc & scc_infs_alone)
-              final[e] = true;
+            final[e] = true;
           else
-              unknown.insert(e);
-        });
+            unknown.insert(e);
+        }
 
       // Erase edges that cannot belong to a cycle, i.e., the edges
       // whose 'dst' is not 'src' of any unknown edges.
@@ -182,13 +163,11 @@ namespace spot
 
             for (unsigned uscc = 0; uscc < si.scc_count(); ++uscc)
               {
-                for_each_edge(aut, si.edges_of(uscc), keep, [&](unsigned e)
-                              {
-                                unknown.erase(e);
-                              });
+                for (auto& e: si.edges_of(uscc))
+                  unknown.erase(aut->edge_number(e));
                 if (si.is_rejecting_scc(uscc))
                   continue;
-                if (!is_scc_tba_type(aut, si, uscc, keep, aut_pairs, final))
+                if (!is_scc_tba_type(aut, si, uscc, aut_pairs, final))
                   return false;
               }
           }
@@ -229,10 +208,9 @@ namespace spot
       scc_info si(aut, scc_info_options::TRACK_STATES);
       std::vector<bool> scc_is_tba_type(si.scc_count(), false);
       std::vector<bool> final(aut->edge_vector().size(), false);
-      std::vector<bool> keep(aut->edge_vector().size(), true);
 
       for (unsigned scc = 0; scc < si.scc_count(); ++scc)
-        scc_is_tba_type[scc] = is_scc_tba_type(aut, si, scc, keep,
+        scc_is_tba_type[scc] = is_scc_tba_type(aut, si, scc,
                                                aut_pairs, final);
 
       auto res = make_twa_graph(aut->get_dict());
@@ -667,6 +645,20 @@ namespace spot
           trace << "main_sets " << main_sets << "\nmain_add "
                 << main_add << '\n';
 
+          // If the SCC is rejecting, there is no need for clone.
+          // Pretend we don't interesect any Fin.
+          if (si.is_rejecting_scc(n))
+            intersects_fin = false;
+
+          // Edges that are already satisfying the acceptance of the
+          // main copy do not need to be duplicated in the clones, so
+          // we fill allacc_edge to remember those.  Of course this is
+          // only needed if the main copy can be accepting and if we
+          // will create clones.
+          std::vector<bool> allacc_edge(aut->edge_vector().size(), false);
+          auto main_acc = res->acc().restrict_to(main_sets | main_add);
+          bool check_main_acc = intersects_fin && !main_acc.is_f();
+
           // Create the main copy
           for (auto s: states)
             for (auto& t: aut->out(s))
@@ -675,11 +667,14 @@ namespace spot
                 if (sbacc || SPOT_LIKELY(si.scc_of(t.dst) == n))
                   a = (t.acc & main_sets) | main_add;
                 res->new_edge(s, t.dst, t.cond, a);
+                // remember edges that are completely accepting
+                if (check_main_acc && main_acc.accepting(a))
+                  allacc_edge[aut->edge_number(t)] = true;
               }
 
           // We do not need any other copy if the SCC is non-accepting,
           // of if it does not intersect any Fin.
-          if (!intersects_fin || si.is_rejecting_scc(n))
+          if (!intersects_fin)
             continue;
 
           // Create clones
@@ -698,7 +693,12 @@ namespace spot
                     auto ns = state_map[s];
                     for (auto& t: aut->out(s))
                       {
-                        if ((t.acc & r) || si.scc_of(t.dst) != n)
+                        if ((t.acc & r) || si.scc_of(t.dst) != n
+                            // edges that are already accepting in the
+                            // main copy need not be copied in the
+                            // clone, since cycles going through them
+                            // are already accepted.
+                            || allacc_edge[aut->edge_number(t)])
                           continue;
                         auto nd = state_map[t.dst];
                         res->new_edge(ns, nd, t.cond, (t.acc & k) | a);
@@ -782,10 +782,9 @@ namespace spot
     // if is TBA type
     scc_info si(aut, scc_info_options::TRACK_STATES);
     std::vector<bool> final(aut->edge_vector().size(), false);
-    std::vector<bool> keep(aut->edge_vector().size(), true);
 
     for (unsigned scc = 0; scc < si.scc_count(); ++scc)
-      if (!is_scc_tba_type(aut, si, scc, keep, aut_pairs, final))
+      if (!is_scc_tba_type(aut, si, scc, aut_pairs, final))
         return false;
 
     return true;
@@ -805,10 +804,9 @@ namespace spot
 
     scc_info si(aut, scc_info_options::TRACK_STATES);
     std::vector<bool> final(aut->edge_vector().size(), false);
-    std::vector<bool> keep(aut->edge_vector().size(), true);
 
     for (unsigned scc = 0; scc < si.scc_count(); ++scc)
-      if (!is_scc_tba_type(aut, si, scc, keep, aut_pairs, final))
+      if (!is_scc_tba_type(aut, si, scc, aut_pairs, final))
         return nullptr;
 
     auto res = make_twa_graph(aut, twa::prop_set::all());
