@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2019 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -23,6 +23,7 @@
 #include <utility>
 #include <cmath>
 #include <limits>
+#include <numeric>
 #include <spot/twaalgos/simulation.hh>
 #include <spot/misc/minato.hh>
 #include <spot/twa/bddprint.hh>
@@ -152,10 +153,12 @@ namespace spot
         return acc_cond::mark_t(res.begin(), res.end());
       }
 
-      direct_simulation(const const_twa_graph_ptr& in)
+      direct_simulation(const const_twa_graph_ptr& in,
+                        std::vector<bdd>* implications = nullptr)
         : po_size_(0),
           all_class_var_(bddtrue),
-          original_(in)
+          original_(in),
+          record_implications_(implications)
       {
         if (!has_separate_sets(in))
           throw std::runtime_error
@@ -212,8 +215,59 @@ namespace spot
         else
           {
             a_ = make_twa_graph(in, twa::prop_set::all());
-            for (auto& t: a_->edges())
-              t.acc ^= all_inf;
+
+            // When we reduce automata with state-based acceptance to
+            // obtain transition-based acceptance, it helps to pull
+            // the state-based acceptance on the incoming edges
+            // instead of the outgoing edges.  A typical example
+            //
+            // ltl2tgba -B GFa | autfilt --small
+            //
+            // The two-state Büchi automaton generated for GFa will
+            // not be reduced to one state if we push the acceptance
+            // marks to the outgoing edges.  However it will be
+            // reduced to one state if we pull the acceptance to the
+            // incoming edges.
+            if (!Sba && !in->prop_state_acc().is_false()
+                && !record_implications_)
+              {
+                // common_out[i] is the set of acceptance set numbers common to
+                // all outgoing edges of state i.
+                std::vector<acc_cond::mark_t> common_out(ns);
+                for (unsigned s = 0; s < ns; ++s)
+                  {
+                    bool first = true;
+                    for (auto& e : a_->out(s))
+                      if (first)
+                        {
+                          common_out[s] = e.acc;
+                          first = false;
+                        }
+                      else if (common_out[s] != e.acc)
+                        {
+                          // If the automaton does not have
+                          // state-based acceptance, do not change
+                          // pull the marks.  Mark the input as
+                          // "not-state-acc" so that we remember
+                          // that.
+                          std::const_pointer_cast<twa_graph>(in)
+                            ->prop_state_acc(false);
+                          goto donotpull;
+                        }
+                  }
+                // Pull the common outgoing sets to the incoming
+                // edges.  Doing so seems to favor cases where states
+                // can be merged.
+                for (auto& e : a_->edges())
+                  e.acc = ((e.acc - common_out[e.src]) | common_out[e.dst])
+                    ^ all_inf;
+              }
+            else
+              {
+              donotpull:
+                for (auto& t: a_->edges())
+                  t.acc ^= all_inf;
+              }
           }
         assert(a_->num_states() == size_a_);
 
@@ -311,10 +365,10 @@ namespace spot
       }
 
       // The core loop of the algorithm.
-      twa_graph_ptr run(std::vector<bdd>* implications = nullptr)
+      twa_graph_ptr run()
       {
         main_loop();
-        return build_result(implications);
+        return build_result();
       }
 
       // Take a state and compute its signature.
@@ -440,8 +494,8 @@ namespace spot
           }
       }
 
-      // Build the minimal resulting automaton.
-      twa_graph_ptr build_result(std::vector<bdd>* implications = nullptr)
+      // Build the simplified automaton.
+      twa_graph_ptr build_result()
       {
         twa_graph_ptr res = make_twa_graph(a_->get_dict());
         res->copy_ap_of(a_);
@@ -456,8 +510,8 @@ namespace spot
 
         auto* gb = res->create_namer<int>();
 
-        if (implications)
-          implications->resize(bdd_lstate_.size());
+        if (record_implications_)
+          record_implications_->resize(bdd_lstate_.size());
         // Create one state per class.
         for (auto& p: sorted_classes_)
           {
@@ -469,10 +523,13 @@ namespace spot
             // update state_mapping
             for (auto& st : p->second)
               (*state_mapping)[st] = s;
-            if (implications)
-              (*implications)[s] = relation_[cl];
+            if (record_implications_)
+              (*record_implications_)[s] = relation_[cl];
           }
 
+        std::vector<bdd> signatures;
+        signatures.reserve(sorted_classes_.size());
+
         // Acceptance of states.  Only used if Sba && Cosimulation.
         std::vector<acc_cond::mark_t> accst;
         if (Sba && Cosimulation)
@@ -485,6 +542,7 @@ namespace spot
         unsigned nb_minato = 0;
 
         auto all_inf = all_inf_;
+        unsigned srcst = 0;
         // For each class, we will create
         // all the edges between the states.
         for (auto& p: sorted_classes_)
@@ -499,7 +557,11 @@ namespace spot
             if (Cosimulation)
               sig = bdd_compose(sig, bddfalse, bdd_var(bdd_initial));
 
-            // Get all the variable in the signature.
+            assert(gb->get_state(src.id()) == srcst);
+            assert(signatures.size() == srcst);
+            signatures.push_back(bdd_exist(sig, all_proms_));
+
+            // Get all the variables in the signature.
             bdd sup_sig = bdd_support(sig);
 
             // Get the variable in the signature which represents the
@@ -554,7 +616,6 @@ namespace spot
                     // acceptance conditions on the input automaton,
                     // we must revert them to create a new edge.
                     acc ^= all_inf;
-
                     if (Cosimulation)
                       {
                         if (Sba)
@@ -564,7 +625,7 @@ namespace spot
                             // to all edges leaving src.  As we
                             // can't do this here, store this in a table
                             // so we can fix it later.
-                            accst[gb->get_state(src.id())] = acc;
+                            accst[srcst] = acc;
                             acc = {};
                           }
                         gb->new_edge(dst.id(), src.id(), cond, acc);
@@ -575,6 +636,7 @@ namespace spot
                       }
                   }
               }
+            ++srcst;
           }
 
         res->set_init_state(gb->get_state(previous_class_
@@ -599,14 +661,86 @@ namespace spot
               }
           }
 
+        // Attempt to merge trivial SCCs
+        if (!record_implications_ && res->num_states() > 1)
+          {
+            scc_info si(res, scc_info_options::NONE);
+            unsigned nscc = si.scc_count();
+            unsigned nstates = res->num_states();
+            std::vector<unsigned> redirect(nstates);
+            std::iota(redirect.begin(), redirect.end(), 0);
+            bool changed = false;
+            for (unsigned scc = 0; scc < nscc; ++scc)
+              if (si.is_trivial(scc))
+                {
+                  unsigned s = si.one_state_of(scc);
+                  bdd ref = signatures[s];
+                  for (unsigned i = 0; i < nstates; ++i)
+                    if (si.reachable_state(i)
+                        && signatures[i] == ref && !si.is_trivial(si.scc_of(i)))
+                      {
+                        changed = true;
+                        redirect[s] = i;
+                        break;
+                      }
+                }
+            if (changed)
+              {
+                if (Cosimulation)
+                  for (auto& e: res->edges())
+                    e.src = redirect[e.src];
+                else
+                  for (auto& e: res->edges())
+                    e.dst = redirect[e.dst];
+                res->set_init_state(redirect[res->get_init_state_number()]);
+              }
+          }
+
         // If we recorded implications for the determinization
         // procedure, we should not remove unreachable states, as that
         // will invalidate the contents of the IMPLICATIONS vector.
         // It's OK not to purge the result, as the determinization
         // will only explore the reachable part anyway.
-        if (!implications)
+        if (!record_implications_)
           res->purge_unreachable_states();
 
+        // Push the common incoming sets to the outgoing edges.
+        // Doing so cancels the preprocessing we did in the other
+        // direction, to prevent marks from moving around the
+        // automaton if we apply simulation several times, and to
+        // favor state-based acceptance.
+        if (!Sba && !Cosimulation && !record_implications_
+            && original_->prop_state_acc())
+          {
+            // common_in[i] is the set of acceptance set numbers
+            // common to all incoming edges of state i.  Only edges
+            // inside one SCC matter.
+            unsigned ns = res->num_states();
+            std::vector<acc_cond::mark_t> common_in(ns);
+            scc_info si(res, scc_info_options::NONE);
+            for (unsigned s = 0; s < ns; ++s)
+              {
+                unsigned s_scc = si.scc_of(s);
+                bool first = true;
+                for (auto& e: res->out(s))
+                  {
+                    if (si.scc_of(e.dst) != s_scc)
+                      continue;
+                    if (first)
+                      {
+                        common_in[s] = e.acc;
+                        first = false;
+                      }
+                    else
+                      {
+                        common_in[s] &= e.acc;
+                      }
+                  }
+              }
+            for (auto& e : res->edges())
+              e.acc = (e.acc - common_in[e.dst]) | common_in[e.src];
+          }
+
         delete gb;
         res->prop_copy(original_,
                        { false, // state-based acc forced below
@@ -709,6 +843,8 @@ namespace spot
       automaton_size stat;
 
       const const_twa_graph_ptr original_;
+
+      std::vector<bdd>* record_implications_;
     };
 
     template<typename Fun, typename Aut>
@@ -745,8 +881,8 @@ namespace spot
              std::vector<bdd>* implications)
   {
     return wrap_simul([implications](const const_twa_graph_ptr& t) {
-                        direct_simulation<false, false> simul(t);
-                        return simul.run(implications);
+                        direct_simulation<false, false> simul(t, implications);
+                        return simul.run();
                       }, t);
   }
 
